#ifndef LLVMHELPERS_H
#define LLVMHELPERS_H

// STL includes
#include <string>

// LLVM includes
#include "llvm/ADT/StringRef.h"
#include "llvm/IR/LegacyPassManager.h"
#include "llvm/Object/Error.h"
#include "llvm/Pass.h"
#include "llvm/Support/Error.h"
#include "llvm/Transforms/IPO/PassManagerBuilder.h"

class PassError : public llvm::ErrorInfo<PassError> {
public:
  static char ID;
  std::string Message;

  explicit PassError(llvm::StringRef Message) : Message(Message.str()) {}

  void log(llvm::raw_ostream &OS) const override { OS << Message; }

  [[nodiscard]] std::error_code convertToErrorCode() const override {
    return make_error_code(llvm::object::object_error::parse_failed);
  }
};

inline char PassError::ID = 42;

template <typename T> class BasicLegacyPassWrapper : public llvm::ModulePass {
public:
  bool runOnModule(llvm::Module &M) override {
    llvm::ModuleAnalysisManager MAM;
    T Pass;
    llvm::PreservedAnalyses Result = Pass.run(M, MAM);
    return !Result.areAllPreserved();
  }

  explicit BasicLegacyPassWrapper(char &pid) : llvm::ModulePass(pid) {}
};

#endif // LLVMHELPERS_H
